/*  FILE:			TWINE.H
	AUTHOR:			Brian A. Lowe, Jr.
	DATE:			March 13, 1995

	OVERVIEW
	========
	Twine object--an efficient and automatic string container class.
	This is the declaration file for the Twine Object, a C++ class for storing, managing and deleting strings.

	This code is given freely to the public domain with no warranties whatsoever. Use it, improve it.
	Contact me on CompuServe at 76640,2664 or Internet at brianlow@mcs.com (preferred) with any
	praise or criticism, bugs or code enhancements you've made. It is some payback for code I've
	downloaded and used.

	Code is set with 4 spaces per tab.

	See TWINE.TXT for usage information and code examples.

*/

#define BOOL	int
#define NULL	0L
#define TRUE	1
#define FALSE	0

class Twine
{
	struct StringHolder					// Structure for the linked list which is the Twine
	{
		long lIndex;					// long index for each string in the Twine
		char far * lpszString;			// long pointer to the string
		StringHolder * pNext;			// Pointer to next StringHolder in the list
	};
	StringHolder * pFirstEntry;			// Pointer to first entry in the list
	StringHolder * pLastEntry;			// Pointer to always empty last (and next) entry in the list
public:
	// Contructor; creates empty Twine; returns NULL if memory allocation error
	Twine();

	// Destructor; deletes memory associated with any strings in Twine
	~Twine();

	// Add string to Twine at end of list
	// Returns pLastEntry->lIndex, -1L if memory allocation error
	long Add (char far * lpszStringToAdd, BOOL bExclusive = TRUE);

	// Add string to Twine at a particular index location; indices of strings after insertion point
	//	are incremented by one
	// Returns pLastEntry->lIndex, -1L if memory allocation error
	long Add (long lAddAtIndex, char far *  lpszStringToAdd, BOOL bExclusive = TRUE);

	// Remove a particular string from list; indices of strings after removed one
	//	are decremented by one.
	// Returns TRUE on success, FALSE on failure
	BOOL Remove (char far * lpszStringToRemove);

	// Remove whatever string is at index location in list; indices of strings after insertion point
	//	are decremented by one
	// Returns TRUE on success, FALSE on failure
	BOOL Remove (long lRemoveAtIndex);

	// Replace the (first occurrence of if non-exclusive) given string with another, no matter what
	//	the index.
	// Returns TRUE on success, FALSE on failure
	BOOL Replace (char far * lpszStringToReplace, char far * lpszNewString);

	// Replace the string at a specific index with the given string
	// Returns TRUE on success, FALSE on failure
	BOOL Replace (char far * lpszNewString, long lReplaceAtIndex);

	// Returns the index of the (first occurrence of if non-exclusive) given string, -1L if not found in Twine
	long Contains (char far * lpszStringToFind);

	// Clears the contents of Twine; deletes all memory associated with all strings in Twine
	void Clear();

	// Returns the number of strings stored in the Twine
	long HowMany() {return pLastEntry->lIndex;}

	// Use like an array subscript to return the string at a given index. Especially useful in doing string
	//	operations in a loop.
	char far * operator[] (long lIndexToFind);
};

